//------------------------------------------------------------//
//                                                            //
//   PROJECT:  food3                                          //
//                                                            //
//   FILE:     food3.cpp                                      //
//   AUTHOR:   saulukas                                       //
//                                                            //
//   New topics:                                              //
//   - keyboard                                               //
//   - containers: vector, list and map                       //
//   - iterator                                               //
//                                                            //
//------------------------------------------------------------//

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <list>
#include <map>

using namespace std;

//------------------------------------------------------------
class Food
{
protected:

   string name;
   double price;

public:

   Food (string name, double price);

   string getName    ()         {return name;} 
   double getPrice   ()         {return price;}
   void   setPrice   (double p) {price = p;}
   
   void   printPrice (double amount);
};

//------------------------------------------------------------
Food::Food (string n, double p)
{
   name  = n;
   price = p;
}

//------------------------------------------------------------
void Food::printPrice (double amount)
{
   cout << "  " << amount << " units of " << name
        << " costs " << (price * amount) << endl;
}

//============================================================//
//                                                            //
//    main ()                                                 //
//                                                            //
//============================================================//
int main ()
{
   vector<Food> foodVector;
   list  <Food> foodList;
   
   for (;;)
   {
      string name;
      double price;
      
      cout << "Enter food name ('q' - quit): ";
      cin >> name;
      if (name == "q") break;
      cout << "Price of one unit of " << name << ": ";
      cin >> price;

      Food food(name, price);
      foodVector.push_back(food);
      foodList  .push_back(food);
   }
   
   cout << "Vector size=" << foodVector.size()
        << " capacity="   << foodVector.capacity() << endl;
   for (int i = 0;  i < foodVector.size();  i++)
      foodVector[i].printPrice(2.00);

   cout << "List size=" << foodList.size() << endl;
   list<Food>::iterator li = foodList.begin();
   for (;  li != foodList.end();  li++)
      (*li).printPrice(3.00);     // not so nice style

   cout << "Iterating through vector:" << endl;
   vector<Food>::iterator vi = foodVector.begin();
   for (;  vi != foodVector.end();  vi++)
      cout << vi->getName () << " "    // better this way 
           << vi->getPrice() << endl;
           
   map<string, int>  wordCount;
   string            word;
   ifstream          inFile ("food3.cpp");
   while (inFile.good())
   {
       inFile >> word;
       wordCount[word] += 1;
   }
   inFile.close();
   cout << "Count of different words: " << wordCount.size() << endl;
   cout << "Contains 'int' = " 
        << (wordCount.find("int")  != wordCount.end()) << endl
        << "Contains 'iint' = " 
        << (wordCount.find("iint") != wordCount.end()) << endl;
        
   wordCount.insert( map<string,int>::value_type("!!!first", 1) );     
   wordCount.insert( map<string,int>::value_type("~~~last",  1) );     
        
   ofstream outFile ("wordCount.txt");
   outFile << "Count of different words: " << wordCount.size() << endl;
   map<string, int>::iterator mi = wordCount.begin();
   for (;  mi != wordCount.end();  mi++)
      outFile << mi->first << " " << mi->second << endl;
   outFile.close();   
}
//============================================================//
